@@ -12,39 +12,48 @@
 #' @slot CAA An array for the predicted catch at age with dimension \code{OM@@nsim}, \code{OM@@nyears}, \code{OM@@maxage}, and nfleet.
 #' @slot CAL An array for the predicted catch at length with dimension \code{OM@@nsim}, \code{OM@@nyears}, length bins, and nfleet.
 #' @slot conv A logical vector of length \code{OM@@nsim} indicating convergence of the RCM in the i-th simulation.
-#' @slot Misc A list of length \code{OM@@nsim} with more output from the fitted RCM. Within each simulation, items of interest include:
+#' @slot Misc A list of length \code{OM@@nsim} with more output from the fitted RCM. Within each simulation, a named list containing items of interest include:
 #'
 #' \itemize{
 #' \item B - total biomass - vector of length nyears+1
 #' \item E0 - annual unfished spawning biomass - vector of length nyears
 #' \item E0_SR - unfished spawning biomass for the stock-recruit relationship - numeric
+#' \item CR_SR - compensation ratio correpsponding to Arec and unfished spawning biomass for the stock-recruit relationship - numeric
 #' \item CR - annual compensation ratio - vector of length nyears
 #' \item Arec - alpha parameter of the stock-recruit relationship - numeric
 #' \item Brec - beta parameter of the stock-recruit relationship - numeric
 #' \item R - recruitment - vector of length nyears+1
-#' \item R_early - recruitment for the cohorts in first year of the model - vector maxage-1
+#' \item R_early - recruitment for the cohorts in first year of the model - vector n_age-1 (where n_age = maxage + 1)
 #' \item VB - vulnerable biomass - matrix of nyears x nfleet
-#' \item N - abundance at age - matrix of nyears+1 x maxage
+#' \item N - abundance at age - matrix of nyears+1 x n_age
 #' \item F - apical fishing mortality - matrix of nyears x nfleet
-#' \item F_at_age - fishing mortality at age - array of nyears x maxage x nfleet
+#' \item F_at_age - fishing mortality at age - matrix of nyears x n_age
 #' \item F_equilibrium - equilibrium fishing mortality prior to first year - vector of length nfleet
-#' \item M - natural mortality - matrix of nyears x maxage
-#' \item Z - total mortality - matrix of nyears x maxage
-#' \item q - survey catchability - vector of length nsurvey
-#' \item s_vul - survey selectivity at age - array of dim nyears+1, maxage, nsurvey
-#' \item s_vul_len - corresponding survey selectivity at length - matrix of nbins x nsurvey
+#' \item M - natural mortality - matrix of nyears x n_age
+#' \item Z - total mortality - matrix of nyears x n_age
+#' \item q - index catchability - vector of length nsurvey
+#' \item ivul - index selectivity at age - array of dim nyears+1, n_age, nsurvey
+#' \item ivul_len - corresponding index selectivity at length - matrix of nbins x nsurvey
 #' \item Ipred - predicted index values - matrix of nyears x nsurvey
-#' \item s_CAApred - predicted survey catch at age - array of dim nyears, maxage, nsurvey
-#' \item vul - fleet selectivity at age - array of dim nyears+1, maxage, nfleet (or nsel_block)
+#' \item IAApred - predicted index catch at age - array of dim nyears, n_age, nsurvey
+#' \item vul - fleet selectivity at age - array of dim nyears+1, n_age, nfleet (or nsel_block)
 #' \item vul_len - corresponding fleet selectivity at length - matrix of nbins x nfleet (or nsel_block)
-#' \item s_CALpred - predicted survey catch at length - array of dim nyears, nbins, nsurvey
+#' \item IALpred - predicted index catch at length - array of dim nyears, nbins, nsurvey
 #' \item MLpred - predicted mean length - matrix of nyears x nfleet
 #' \item MWpred - predicted mean weight - matrix of nyears x nfleet
-#' \item CAApred - predicted catch at age - array of nyears, maxage, nfleet
+#' \item CAApred - predicted catch at age - array of nyears, n_age, nfleet
 #' \item CALpred - predicted catch at length - array of nyears, nbins, nfleet
 #' \item Cpred - predicted catch in weight - matrix of nyears x nfleet
 #' \item CN - predicted catch in numbers - matrix of nyears x nfleet
-#' \item nll - Total objective function of the model - numeric
+#' \item dynamic_SSB0 - the dynamic unfished spawning biomass calcaluated by projecting the historical model with zero catches - vector of length nyears+1
+#' \item SPR_eq - equilibrium spawning potential ratio calculated from annual F-at-age - vector of length nyears
+#' \item SPR_dyn - dynamic (transitional) spawning potential ratio calculated from cumulative survival of cohorts - vector of length nyears
+#' \item nll - total objective function of the model - numeric
+#' \item nll_fleet - objective function values for each annual data point(s) from fleets - array of nyears x nfleet x 5 (for Catch, equilibrium catch, CAA, CAL, and mean size)
+#' \item nll_index - objective function values for each annual data point(s) in the index - array of nyears x nsurvey x 3 (for Index, IAA, and IAL)
+#' \item prior - penalty value added to the objective function from priors - numeric
+#' \item penalty - additional penalty values added to the objective function due to high F - numeric
+#' \item conv - whether the model converged (whether a positive-definite Hessian was obtained) - logical
 #' }
 #'
 #' @slot mean_fit A list of output from fit to mean values of life history parameters in the operating model. The named list consists of:
@@ -56,7 +65,7 @@
 #' \code{\link[TMB]{sdreport}}.
 #' \item report - a list of model output reported from the TMB executable, i.e. \code{obj$report()}. See Misc.
 #' }
-#' @slot data A list of the data inputs for the RCM.
+#' @slot data A \linkS4class{RCMdata} object containing data inputs for the RCM.
 #' @slot config A data frame describing configuration of the RCM (not currently used).
 #'
 #' @seealso \link{plot.RCModel} \link{RCM}
@@ -65,7 +74,7 @@
 #' @exportClass RCModel
 RCModel <- setClass("RCModel", slots = c(OM = "ANY", SSB = "matrix", NAA = "array",
                                          CAA = "array", CAL = "array", conv = "logical", Misc = "list", mean_fit = "list",
-                                         data = "list", config = "data.frame"))
+                                         data = "ANY", config = "data.frame"))
 
 
 #' @name plot.RCModel
@@ -99,7 +108,7 @@ RCModel <- setClass("RCModel", slots = c(OM = "ANY", SSB = "matrix", NAA = "arra
 #' @seealso \linkS4class{RCModel} \link{RCM}
 #' @exportMethod plot
 setMethod("plot", signature(x = "RCModel", y = "missing"),
-          function(x, compare = TRUE, filename = "RCM", dir = tempdir(), sims = 1:x@OM@nsim, Year = NULL,
+          function(x, compare = FALSE, filename = "RCM", dir = tempdir(), sims = 1:x@OM@nsim, Year = NULL,
                    f_name = NULL, s_name = NULL, MSY_ref = c(0.5, 1), bubble_adj = 10, scenario = list(), title = NULL,
                    open_file = TRUE, quiet = TRUE, render_args, ...) {
 
@@ -145,11 +154,11 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
 
             ####### Assign variables
             OM <- MSEtool::SubCpars(x@OM, sims)
-            mean_fit <- x@mean_fit
             report_list <- x@Misc[sims]
 
             nsim <- OM@nsim
-            data <- x@data
+            RCMdata <- x@data
+            stopifnot(inherits(RCMdata, "RCMdata"))
 
             max_age <- OM@maxage
             age <- 0:max_age
@@ -158,23 +167,17 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
             if(is.null(Year)) Year <- (OM@CurrentYr - nyears + 1):OM@CurrentYr
             Yearplusone <- c(Year, max(Year) + 1)
 
-            # Backwards compatibility
-            if(is.null(data$nsel_block)) {
-              data$nsel_block <- data$nfleet
-              data$sel_block <- matrix(1:data$nfleet, nyears, data$nfleet, byrow = TRUE)
-            }
-
-            nfleet <- data$nfleet
-            nsel_block <- data$nsel_block
-            nsurvey <- data$nsurvey
-            length_bin <- data$length_bin
+            nfleet <- RCMdata@Misc$nfleet
+            nsel_block <- RCMdata@Misc$nsel_block
+            nsurvey <- RCMdata@Misc$nsurvey
+            length_bin <- RCMdata@Misc$lbinmid
 
             if(is.null(f_name)) f_name <- paste("Fleet", 1:nfleet)
             if(is.null(s_name)) {
               if(nsurvey > 0) {
-                s_name <- paste("Survey", 1:nsurvey)
+                s_name <- paste("Index", 1:nsurvey)
               } else {
-                s_name <- "Survey"
+                s_name <- "Index"
               }
             }
 
@@ -198,9 +201,6 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
                         "```\n")
 
             ####### Updated historical OM parameters
-            #Year_matrix <- matrix(Year, ncol = nsim, nrow = nyears)
-            #Yearplusone_matrix <- matrix(Yearplusone, ncol = nsim, nrow = nyears+1)
-
             OM_update <- c("# Summary {.tabset}\n",
                            "## Updated historical OM parameters\n", rmd_RCM_R0(),
                            rmd_RCM_D(), rmd_RCM_Perr(), rmd_RCM_Find(), rmd_RCM_sel())
@@ -208,39 +208,31 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
             ####### Output from all simulations {.tabset}
             fleet_output <- lapply(1:nfleet, rmd_RCM_fleet_output, f_name = f_name)
 
-            if(any(data$Index > 0, na.rm = TRUE)) {
-              survey_output <- lapply(1:nsurvey, rmd_RCM_survey_output, s_name = s_name)
-            } else survey_output <- NULL
+            if(any(RCMdata@Index > 0, na.rm = TRUE)) {
+              index_output <- lapply(1:nsurvey, rmd_RCM_index_output, s_name = s_name)
+            } else index_output <- NULL
 
-            all_sims_output <- c(fleet_output, survey_output, "### Model predictions\n",
-                                 rmd_RCM_initD(), rmd_RCM_R_output(), rmd_RCM_SSB_output(), rmd_log_rec_dev())
+            all_sims_output <- c(fleet_output, index_output, "### Model predictions\n",
+                                 rmd_RCM_initD(), rmd_RCM_R_output(), rmd_RCM_SSB_output(), rmd_log_rec_dev(), 
+                                 rmd_RCM_SPR())
 
             ####### Fit to mean inputs from operating model
             # Generate summary table (parameter estimates)
 
-            if(length(x@mean_fit) > 0) {
-              SD <- x@mean_fit$SD
+            if(length(x@mean_fit)) {
               report <- x@mean_fit$report
-              data_mean_fit <- x@mean_fit$obj$env$data
-              length_bin <- data_mean_fit$length_bin
-
-              # Backwards compatibility
-              if(is.null(data_mean_fit$nsel_block)) {
-                data_mean_fit$nsel_block <- data$nfleet
-                data_mean_fit$sel_block <- matrix(1:data$nfleet, nyears, data$nfleet, byrow = TRUE)
-              }
-
               conv <- report$conv
+              data_mean_fit <- x@mean_fit$obj$env$data
+              SD <- x@mean_fit$SD
 
-              SD2 <- sdreport_int(SD) %>% signif(4) %>% format() %>% as.data.frame()
               if(render_args$output_format == "html_document") {
                 sumry <- c("## Fit to mean parameters of the OM {.tabset}\n",
                            "### RCM Estimates\n",
-                           "`r SD2`\n\n")
+                           "`r sdreport_int(SD) %>% signif(4) %>% format() %>% as.data.frame()`\n\n")
               } else {
                 sumry <- c("## Fit to mean parameters of the OM {.tabset}\n",
                            "### RCM Estimates\n",
-                           "`r SD2 %>% knitr::kable(format = \"markdown\")`\n\n")
+                           "`r sdreport_int(SD) %>% signif(4) %>% format() %>% as.data.frame() %>% knitr::kable(format = \"markdown\")`\n\n")
               }
 
               # Life History section
@@ -248,23 +240,28 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
                 n_unique <- apply(x, 2, function(y) length(unique(y)))
                 any(n_unique > 1)
               }
-              LAA <- rmd_at_age(age, data_mean_fit$len_age[nyears, ], header = "### Life History\n", fig.cap = "Length-at-age in last historical year.",
-                                label = "Mean Length-at-age")
+              if(any(data_mean_fit$MLpred > 0) && any(data_mean_fit$CALpred > 0)) {
+                SD_LAA <- "data_mean_fit$SD_LAA[nyears, ]"
+              } else {
+                SD_LAA <- ""
+              }
+              LAA <- rmd_LAA(age = "age", LAA = "data_mean_fit$len_age[nyears, ]", header = "### Life History\n", 
+                             SD_LAA = SD_LAA, fig.cap = "Length-at-age in last historical year.")
               if(LH_varies_fn(data_mean_fit$len_age)) {
                 LAA_persp <- rmd_persp_plot(x = "Year", y = "age", z = "data_mean_fit$len_age[1:nyears, ]", xlab = "Year", ylab = "Age",
                                             zlab = "Length-at-age", phi = 35, theta = 45, expand = 0.55, fig.cap = "Annual length-at-age.")
               } else LAA_persp <- NULL
 
-              mat <- rmd_mat(age, data_mean_fit$mat[nyears, ], fig.cap = "Maturity-at-age in last historical year.")
+              mat <- rmd_mat(age = "age", "data_mean_fit$mat[nyears, ]", fig.cap = "Maturity-at-age in last historical year.")
               if(LH_varies_fn(data_mean_fit$mat)) {
                 mat_persp <- rmd_persp_plot(x = "Year", y = "age", z = "data_mean_fit$mat[1:nyears, ]", xlab = "Year", ylab = "Age",
                                             zlab = "Maturity-at-age", phi = 35, theta = 45, expand = 0.55, fig.cap = "Annual maturity-at-age.")
               } else mat_persp <- NULL
 
               if(data_mean_fit$use_prior[3]) {
-                NatM <- rmd_at_age(age, rep(report$Mest, length(age)), fig.cap = "Natural mortality (time constant).", label = "Natural mortality")
+                NatM <- rmd_at_age(age = "age", "rep(report$Mest, length(age))", fig.cap = "Natural mortality (time constant).", label = "Natural mortality")
               } else {
-                NatM <- rmd_at_age(age, data_mean_fit$M[nyears, ], fig.cap = "Natural mortality in last historical year.", label = "Natural mortality")
+                NatM <- rmd_at_age(age = "age", "data_mean_fit$M[nyears, ]", fig.cap = "Natural mortality in last historical year.", label = "Natural mortality")
               }
               if(!data_mean_fit$use_prior[3] && LH_varies_fn(data_mean_fit$M)) {
                 NatM_persp <- rmd_persp_plot(x = "Year", y = "age", z = "data_mean_fit$M[1:nyears, ]", xlab = "Year", ylab = "Age",
@@ -298,122 +295,113 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
                   rr2 <- rmd_fit_comps("Year", obs2, pred2, type = "bubble_residuals", ages = "age", fig.cap = fig.cap2)
                   rr3 <- rmd_fit_comps("Year", obs2, pred2, type = "mean", ages = "age", fig.cap = fig.cap3)
                 } else {
-                  rr <- rmd_fit_comps("Year", obs2, pred2, type = "annual", CAL_bins = "data$length_bin", fig.cap = fig.cap)
-                  rr2 <- rmd_fit_comps("Year", obs2, pred2, type = "bubble_residuals", CAL_bins = "data$length_bin", fig.cap = fig.cap2)
-                  rr3 <- rmd_fit_comps("Year", obs2, pred2, type = "mean", CAL_bins = "data$length_bin", fig.cap = fig.cap3)
+                  rr <- rmd_fit_comps("Year", obs2, pred2, type = "annual", CAL_bins = "RCMdata@Misc$lbinmid", fig.cap = fig.cap)
+                  rr2 <- rmd_fit_comps("Year", obs2, pred2, type = "bubble_residuals", CAL_bins = "RCMdata@Misc$lbinmid", fig.cap = fig.cap2)
+                  rr3 <- rmd_fit_comps("Year", obs2, pred2, type = "mean", CAL_bins = "RCMdata@Misc$lbinmid", fig.cap = fig.cap3)
                 }
                 c(rr, rr2, rr3)
               }
 
-              if(any(data$Chist > 0, na.rm = TRUE)) {
-                C_matplot <- rmd_matplot(x = "matrix(Year, nyears, nfleet)", y = "data$Chist", col = "rich.colors(nfleet)",
+              if(any(RCMdata@Chist > 0, na.rm = TRUE)) {
+                C_matplot <- rmd_matplot(x = "Year", y = "RCMdata@Chist", col = "rich.colors(nfleet)",
                                          xlab = "Year", ylab = "Catch", legend.lab = "f_name",
                                          fig.cap = "Catch time series.", header = "### Data and Fit {.tabset}\n#### Catch \n")
 
-                if(data_mean_fit$condition == "effort" || ncol(data$Chist) > 1) {
-                  C_plots <- lapply(1:nfleet, individual_matrix_fn, obs = "data$Chist", pred = "report$Cpred",
+                if(data_mean_fit$condition == "effort" || ncol(RCMdata@Chist) > 1) {
+                  C_plots <- lapply(1:nfleet, individual_matrix_fn, obs = "RCMdata@Chist", pred = "report$Cpred",
                                     fig.cap = "catch from fleet", label = f_name, match = data_mean_fit$condition == "catch2")
                 } else C_plots <- NULL
               } else C_matplot <- C_plots <- NULL
 
-              if(any(data$Ehist > 0, na.rm = TRUE)) {
+              if(any(RCMdata@Ehist > 0, na.rm = TRUE)) {
                 if(is.null(C_matplot)) {
                   E_header <- "### Data and Fit {.tabset}\n#### Effort \n"
                 } else {
                   E_header <- "#### Effort \n"
                 }
-                E_matplot <- rmd_matplot(x = "matrix(Year, nyears, nfleet)", y = "data_mean_fit$E_hist", col = "rich.colors(nfleet)",
+                E_matplot <- rmd_matplot(x = "Year", y = "data_mean_fit$E_hist", col = "rich.colors(nfleet)",
                                          xlab = "Year", ylab = "Effort", legend.lab = "f_name", fig.cap = "Effort time series.", header = E_header)
               } else E_matplot <- NULL
 
-              if(any(data$Index > 0, na.rm = TRUE)) {
-                I_plots <- c("#### Surveys \n",
-                             lapply(1:nsurvey, individual_matrix_fn, obs = "data$Index", pred = "report$Ipred",
+              if(any(RCMdata@Index > 0, na.rm = TRUE)) {
+                I_plots <- c("#### Index \n",
+                             lapply(1:nsurvey, individual_matrix_fn, obs = "RCMdata@Index", pred = "report$Ipred",
                                     fig.cap = "index from survey", label = s_name),
-                             lapply(1:nsurvey, individual_matrix_fn, obs = "data$Index", pred = "report$Ipred",
+                             lapply(1:nsurvey, individual_matrix_fn, obs = "RCMdata@Index", pred = "report$Ipred",
                                     fig.cap = "index from survey", label = paste(s_name, "Residuals"), resids = TRUE))
               } else I_plots <- NULL
 
-              if(any(data$CAA > 0, na.rm = TRUE)) {
+              if(any(RCMdata@CAA > 0, na.rm = TRUE)) {
                 CAA_plots <- c("#### Age comps \n",
-                               lapply(1:nfleet, individual_array_fn, obs = "data$CAA", pred = "report$CAApred", comps = "age", label = f_name))
+                               lapply(1:nfleet, individual_array_fn, obs = "RCMdata@CAA", pred = "report$CAApred", comps = "age", label = f_name))
               } else CAA_plots <- NULL
 
-              if(any(data$CAL > 0, na.rm = TRUE)) {
+              if(any(RCMdata@CAL > 0, na.rm = TRUE)) {
                 CAL_plots <- c("#### Length comps \n",
-                               lapply(1:nfleet, individual_array_fn, obs = "data$CAL", pred = "report$CALpred", comps = "length", label = f_name))
+                               lapply(1:nfleet, individual_array_fn, obs = "RCMdata@CAL", pred = "report$CALpred", comps = "length", label = f_name))
               } else CAL_plots <- NULL
 
-              if(any(data$MS > 0, na.rm = TRUE)) {
-                if(data$MS_type == "length") {
+              if(any(RCMdata@MS > 0, na.rm = TRUE)) {
+                if(RCMdata@MS_type == "length") {
                   MS_label <- paste("Mean Length from", f_name)
                 } else {
                   MS_label <- paste("Mean Weight from", f_name)
                 }
-                MS_plots <- c(paste0("#### Mean ", data$MS_type, "\n"),
-                              lapply(1:nfleet, individual_matrix_fn, obs = "data$MS",
-                                     pred = ifelse(data$MS_type == "length", "report$MLpred", "report$MWpred"),
-                                     fig.cap = paste("mean", data$MS_type, "from fleet"), label = MS_label))
+                MS_plots <- c(paste0("#### Mean ", RCMdata@MS_type, "\n"),
+                              lapply(1:nfleet, individual_matrix_fn, obs = "RCMdata@MS",
+                                     pred = ifelse(RCMdata@MS_type == "length", "report$MLpred", "report$MWpred"),
+                                     fig.cap = paste("mean", RCMdata@MS_type, "from fleet"), label = MS_label))
               } else MS_plots <- NULL
 
-              if(any(data$s_CAA > 0, na.rm = TRUE)) {
-                s_CAA_plots <- c("#### Survey age comps \n",
-                                 lapply(1:nsurvey, individual_array_fn, obs = "data$s_CAA", pred = "report$s_CAApred", comps = "age", label = s_name))
-              } else s_CAA_plots <- NULL
+              if(any(RCMdata@IAA > 0, na.rm = TRUE)) {
+                IAA_plots <- c("#### Index age comps \n",
+                                 lapply(1:nsurvey, individual_array_fn, obs = "RCMdata@IAA", pred = "report$IAApred", comps = "age", label = s_name))
+              } else IAA_plots <- NULL
 
-              if(any(data$s_CAL > 0, na.rm = TRUE)) {
-                s_CAL_plots <- c("#### Survey length comps \n",
-                                 lapply(1:nsurvey, individual_array_fn, obs = "data$s_CAL", pred = "report$s_CALpred", comps = "length", label = s_name))
-              } else s_CAL_plots <- NULL
+              if(any(RCMdata@IAL > 0, na.rm = TRUE)) {
+                IAL_plots <- c("#### Index length comps \n",
+                                 lapply(1:nsurvey, individual_array_fn, obs = "RCMdata@IAL", pred = "report$IALpred", comps = "length", label = s_name))
+              } else IAL_plots <- NULL
 
-              data_section <- c(C_matplot, C_plots, E_matplot, I_plots, CAA_plots, CAL_plots, MS_plots, s_CAA_plots, s_CAL_plots)
+              data_section <- c(C_matplot, C_plots, E_matplot, I_plots, CAA_plots, CAL_plots, MS_plots, IAA_plots, IAL_plots)
 
               # Model output
-              sel_matplot <- rmd_matplot(x = "matrix(age, max_age + 1, nfleet)", y = "matrix(report$vul[nyears, , ], max_age + 1, nfleet)", col = "rich.colors(nfleet)",
+              sel_matplot <- rmd_matplot(x = "age", y = "matrix(report$vul[nyears, , ], max_age + 1, nfleet)", col = "rich.colors(nfleet)",
                                          xlab = "Age", ylab = "Selectivity", legend.lab = "f_name",
                                          fig.cap = "Terminal year selectivity by fleet.", header = "### Output \n")
 
-              F_matplot <- rmd_matplot(x = "matrix(Year, nyears, nfleet)", y = "report$F", col = "rich.colors(nfleet)",
+              F_matplot <- rmd_matplot(x = "Year", y = "report$F", col = "rich.colors(nfleet)",
                                        xlab = "Year", ylab = "Fishing Mortality (F)", legend.lab = "f_name",
                                        fig.cap = "Time series of fishing mortality by fleet.")
 
-              SSB <- structure(report$E, names = c(Year, max(Year) + 1))
-              dynamic_SSB0 <- structure(report$dynamic_SSB0, names = names(SSB))
-              SSB0 <- structure(report$E0, names = Year)
-              if(length(unique(SSB0)) > 1) {
-                SSB_plot <- rmd_assess_timeseries("SSB0", "unfished spawning depletion (growth and/or M are time-varying)",
-                                                  "expression(SSB[0])")
-              } else SSB_plot <- NULL
+              if(length(unique(report$E0)) > 1) {
+                SSB0_plot <- rmd_assess_timeseries("structure(report$E0, names = Year)", "unfished spawning depletion (growth and/or M are time-varying)",
+                                                   "expression(SSB[0])")
+              } else SSB0_plot <- NULL
 
-              SSB_SSB0 <- structure(report$E/report$E0_SR, names = c(Year, max(Year) + 1))
+              N_bubble <- rmd_bubble("Yearplusone", "report$N", ages = "age", fig.cap = "Predicted abundance-at-age.")
+              CAA_bubble <- rmd_bubble("Year", "apply(report$CAApred, 1:2, sum)", ages = "age",
+                                       fig.cap = "Predicted catch-at-age (summed over all fleets).", 
+                                       bubble_adj = as.character(bubble_adj))
 
-              R <- structure(report$R, names = c(Year, max(Year) + 1))
-              N_at_age <- report$N
-              N <- structure(rowSums(N_at_age), names = c(Year, max(Year) + 1))
-
-              log_rec_dev <- structure(report$log_rec_dev, names = Year)
-              log_rec_dev_SE <- summary(SD)[, 2]
-              log_rec_dev_SE <- log_rec_dev_SE[names(log_rec_dev_SE) == "log_rec_dev"]
-              log_rec_dev_SE <- ifelse(data_mean_fit$est_rec_dev, log_rec_dev_SE, 0)
-
-              N_bubble <- rmd_bubble("c(Year, max(Year)+1)", "N_at_age", fig.cap = "Predicted abundance-at-age.")
-
-              CAA_all <- apply(report$CAApred, c(1, 2), sum)
-              CAA_bubble <- rmd_bubble("Year", "CAA_all", fig.cap = "Predicted catch-at-age (summed over all fleets).", bubble_adj = as.character(bubble_adj))
-
-              CAL_all <- apply(report$CALpred, c(1, 2), sum)
-              if(sum(CAL_all, na.rm = TRUE) > 0) {
-                CAL_bubble <- rmd_bubble("Year", "CAL_all", CAL_bins = "data_mean_fit$length_bin",
+              if(sum(report$CALpred, na.rm = TRUE) > 0) {
+                CAL_bubble <- rmd_bubble("Year", "apply(report$CALpred, 1:2, sum)", CAL_bins = "RCMdata@Misc$lbinmid",
                                          fig.cap = "Predicted catch-at-length (summed over all fleets).", bubble_adj = as.character(bubble_adj))
-              } else CAL_bubble <- ""
+              } else CAL_bubble <- NULL
 
-              ts_output <- c(sel_matplot, F_matplot, rmd_SSB(), SSB_plot, rmd_SSB_SSB0(FALSE), rmd_dynamic_SSB0(), rmd_R(), rmd_RCM_SR(),
-                             rmd_residual("log_rec_dev", fig.cap = "Time series of recruitment deviations.", label = "log-Recruitment deviations"),
-                             rmd_residual("log_rec_dev", "log_rec_dev_SE", fig.cap = "Time series of recruitment deviations with 95% confidence intervals.",
+              ts_output <- c(sel_matplot, F_matplot, rmd_SSB("structure(report$E, names = Yearplusone)"), 
+                             SSB0_plot, rmd_SSB_SSB0(FALSE, "structure(report$E/report$E0_SR, names = Yearplusone)"), 
+                             rmd_dynamic_SSB0("structure(report$dynamic_SSB0, names = Yearplusone)"), 
+                             rmd_R("structure(report$R, names = Yearplusone)"), rmd_RCM_SR(), rmd_RCM_SPR2(),
+                             rmd_residual("structure(report$log_rec_dev, names = Year)", fig.cap = "Time series of recruitment deviations.", label = "log-Recruitment deviations"),
+                             rmd_residual("structure(report$log_rec_dev, names = Year)", 
+                                          "ifelse(data_mean_fit$est_rec_dev, as.list(SD, \"Std. Error\")$log_rec_dev, 0)", 
+                                          fig.cap = "Time series of recruitment deviations with 95% confidence intervals.",
                                           label = "log-Recruitment deviations", conv_check = TRUE),
-                             rmd_N(), N_bubble, CAA_bubble, CAL_bubble)
+                             rmd_N("structure(rowSums(report$N), names = Yearplusone)"), 
+                             N_bubble, CAA_bubble, CAL_bubble)
 
-              nll <- RCM_get_likelihoods(report, data$LWT, f_name, s_name)
+              nll <- RCM_get_likelihoods(report, RCMdata@Misc$LWT, f_name, s_name)
               
               nll_table <- c("### Likelihood components\n",
                              "#### Summary\n",
@@ -431,7 +419,7 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
                                "`r SD$env$corr.fixed %>% structure(dimnames = list(make_unique_names(rownames(.)), make_unique_names(colnames(.)))) %>% as.data.frame()`\n\n")
               
               if(is.array(report$nll_fleet)) {
-                like_gradients <- c("### Likelihood gradients {.tabset}\n", rmd_RCM_likelihood_gradients(f_name, s_name, do_survey = any(data$Index > 0, na.rm = TRUE)))
+                like_gradients <- c("### Likelihood gradients {.tabset}\n", rmd_RCM_likelihood_gradients(f_name, s_name, do_index = any(RCMdata@Index > 0, na.rm = TRUE)))
               } else {
                 like_gradients <- NULL
               }
@@ -441,15 +429,19 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
               } else ret <- NULL
 
               mean_fit_rmd <- c(sumry, LH_section, data_section, ts_output, nll_table, corr_matrix, like_gradients, ret)
-            } else mean_fit_rmd <- c("## Fit to mean parameters of OM {.tabset}\n",
-                                     "No model found. Re-run `RCM()` with `mean_fit = TRUE`.\n\n")
+            } else {
+              mean_fit_rmd <- c("## Fit to mean parameters of OM {.tabset}\n",
+                                "No model found. Re-run `RCM()` with `mean_fit = TRUE`.\n\n")
+            }
 
             if(compare) {
               message("Getting Hist object from runMSE...")
               Hist <- runMSE(OM, Hist = TRUE, silent = TRUE, parallel = OM@nsim >= 48 & snowfall::sfIsRunning())
               compare_rmd <- rmd_RCM_Hist_compare()
-            } else compare_rmd <- c("## Updated OM\n",
-                                    "Re-run `plot()` function with argument `compare = TRUE`.\n\n")
+            } else {
+              compare_rmd <- c("## Updated OM\n",
+                               "Re-run `plot()` function with argument `compare = TRUE`.\n\n")
+            }
 
             rmd <- c(header, OM_update, all_sims_output, mean_fit_rmd, compare_rmd, rmd_footer())
             if(is.list(rmd)) rmd <- do.call(c, rmd)
@@ -477,12 +469,12 @@ setMethod("plot", signature(x = "RCModel", y = "missing"),
 
 #' @rdname plot.RCModel
 #' @export
-compare_RCM <- function(..., compare = TRUE, filename = "compare_RCM", dir = tempdir(), Year = NULL,
+compare_RCM <- function(..., compare = FALSE, filename = "compare_RCM", dir = tempdir(), Year = NULL,
                         f_name = NULL, s_name = NULL, MSY_ref = c(0.5, 1), bubble_adj = 10, scenario = list(), title = NULL,
                         open_file = TRUE, quiet = TRUE, render_args) {
 
   dots <- list(...)
-  test <- vapply(dots, inherits, character(1), what = "RCModel") %>% all()
+  test <- vapply(dots, inherits, logical(1), what = "RCModel") %>% all()
   if(!test) stop("Objects provided are not of class RCModel.", call. = FALSE)
   
   # Update scenario
@@ -517,7 +509,7 @@ compare_RCM <- function(..., compare = TRUE, filename = "compare_RCM", dir = tem
   report_list <- lapply(dots, function(xx) if(length(xx@mean_fit) > 0) return(xx@mean_fit$report) else stop("Error in RCM objects."))
 
   nsim <- length(report_list)
-  data <- dots[[1]]@data
+  RCMdata <- dots[[1]]@data
 
   max_age <- dots[[1]]@OM@maxage
   age <- 0:max_age
@@ -525,18 +517,12 @@ compare_RCM <- function(..., compare = TRUE, filename = "compare_RCM", dir = tem
   if(is.null(Year)) Year <- (dots[[1]]@OM@CurrentYr - nyears + 1):dots[[1]]@OM@CurrentYr
   Yearplusone <- c(Year, max(Year) + 1)
 
-  nfleet <- vapply(dots, function(xx) xx@data$nfleet, numeric(1)) %>% unique()
-  nsurvey <- vapply(dots, function(xx) xx@data$nsurvey, numeric(1)) %>% unique()
-  length_bin <- dots[[1]]@data$length_bin
-
-  # Backwards compatibility
-  if(is.null(data$nsel_block)) {
-    data$nsel_block <- data$nfleet
-    data$sel_block <- matrix(1:data$nfleet, nyears, data$nfleet, byrow = TRUE)
-  }
+  nfleet <- vapply(dots, function(xx) xx@data@Misc$nfleet, numeric(1)) %>% unique()
+  nsurvey <- vapply(dots, function(xx) xx@data@Misc$nsurvey, numeric(1)) %>% unique()
+  length_bin <- RCMdata@Misc$lbinmid
 
   if(is.null(f_name)) f_name <- paste("Fleet", 1:nfleet)
-  if(is.null(s_name)) s_name <- paste("Survey", 1:nsurvey)
+  if(is.null(s_name)) s_name <- paste("Index", 1:nsurvey)
 
   ####### Document header
   if(is.null(title)) title <- "Comparisons of OM conditioning"
@@ -561,8 +547,8 @@ compare_RCM <- function(..., compare = TRUE, filename = "compare_RCM", dir = tem
   ####### Output from all simulations {.tabset}
   fleet_output <- lapply(1:nfleet, rmd_RCM_fleet_output, f_name = f_name)
 
-  if(any(data$Index > 0, na.rm = TRUE)) {
-    survey_output <- lapply(1:nsurvey, rmd_RCM_survey_output, s_name = s_name)
+  if(any(RCMdata@Index > 0, na.rm = TRUE)) {
+    survey_output <- lapply(1:nsurvey, rmd_RCM_index_output, s_name = s_name)
   } else survey_output <- NULL
 
   #### MSY comparisons
@@ -604,10 +590,10 @@ compare_RCM <- function(..., compare = TRUE, filename = "compare_RCM", dir = tem
                        rmd_RCM_initD(), rmd_RCM_R_output(), rmd_RCM_SSB_output(), SSB_MSY, rmd_log_rec_dev())
 
   #### Likelihoods
-  nll <- Map(RCM_get_likelihoods, x = report_list, LWT = lapply(dots, function(xx) xx@data$LWT),
+  nll <- Map(RCM_get_likelihoods, x = report_list, LWT = lapply(dots, function(xx) xx@data@Misc$LWT),
              MoreArgs = list(f_name = f_name, s_name = s_name))
 
-  summary_nll <- vapply(nll, function(xx) xx[[1]] %>% as.matrix(), numeric(4)) %>%
+  summary_nll <- vapply(nll, function(xx) xx[[1]] %>% as.matrix(), numeric(6)) %>%
     structure(dimnames = list(rownames(nll[[1]][[1]]), scenario$names)) %>% as.data.frame()
 
   if(render_args$output_format == "html_document") {
@@ -667,3 +653,5 @@ compare_RCM <- function(..., compare = TRUE, filename = "compare_RCM", dir = tem
   invisible(output_filename)
 }
 
+
+
